package ru.ulstu.extractor.generation.service;

import org.springframework.stereotype.Service;
import ru.ulstu.extractor.branch.model.Branch;
import ru.ulstu.extractor.branch.service.BranchService;
import ru.ulstu.extractor.generation.model.GenerationForm;
import ru.ulstu.extractor.ts.model.TimeSeriesType;
import ru.ulstu.extractor.ts.model.TimeSeriesValue;
import ru.ulstu.extractor.ts.service.TimeSeriesService;
import ru.ulstu.extractor.ts.util.DateUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

@Service
public class GenerationService {
    private final TimeSeriesService timeSeriesService;
    private final BranchService branchService;

    public GenerationService(TimeSeriesService timeSeriesService,
                             BranchService branchService) {
        this.timeSeriesService = timeSeriesService;
        this.branchService = branchService;
    }

    public void generate(GenerationForm generationForm) {
        Branch branch = branchService.findByBranchId(Branch.GENERATED_BRANCH_ID).orElseThrow();
        Date startDate = DateUtils.addDays(-generationForm.getTsLength());
        Arrays.stream(TimeSeriesType.values()).forEach(tsType -> {
            List<TimeSeriesValue> tsValues = new ArrayList<>();
            for (int i = 0; i < generationForm.getTsLength(); i++) {
                tsValues.add(new TimeSeriesValue(DateUtils.addDays(startDate, i), (double) i));
            }
            final String tsName = "Генерированный " + tsType.getDescription();
            timeSeriesService.save(tsName, branch, tsType, tsValues);
        });
    }
}
